package org.bitbucket.bradleysmithllc.etlunit.feature.informatica;

import org.bitbucket.bradleysmithllc.etlunit.*;
import org.bitbucket.bradleysmithllc.etlunit.context.VariableContext;
import org.bitbucket.bradleysmithllc.etlunit.feature.informatica.informatica.ExecuteHandler;
import org.bitbucket.bradleysmithllc.etlunit.feature.informatica.informatica.ExecuteRequest;
import org.bitbucket.bradleysmithllc.etlunit.feature.informatica.util.InformaticaRepositoryClient;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestMethod;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestOperation;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestValueObject;
import org.bitbucket.bradleysmithllc.etlunit.util.IOUtils;

import javax.inject.Inject;
import javax.inject.Named;
import java.io.File;
import java.util.Collection;
import java.util.Map;

public class InformaticaClassListener extends NullClassListener implements ExecuteHandler
{
	private final InformaticaFeatureModule informaticaFeatureModule;
	private final InformaticaExecutor informaticaExecutor;

	private Log applicationLog;

	public InformaticaClassListener(InformaticaFeatureModule informaticaFeatureModule, InformaticaExecutor exe) {
		this.informaticaFeatureModule = informaticaFeatureModule;
		informaticaExecutor = exe;
	}

	@Override
	public void begin(ETLTestMethod mt, VariableContext context) throws TestAssertionFailure, TestExecutionError, TestWarning {
		// before each method, perform a deep cleanse of the working root
		InformaticaConfiguration config = informaticaFeatureModule.getInformaticaConfiguration();

		Map<String, InformaticaDomain> domMap = config.getDomains();

		Collection<InformaticaDomain> domSet = domMap.values();

		for (InformaticaDomain domain : domSet)
		{
			File workingRoot = domain.getLocalizedWorkingRoot();

			if (workingRoot.exists())
			{
				applicationLog.info("Purging remote workspace at path: " + workingRoot);
				IOUtils.purge(workingRoot, true);
				applicationLog.info("Done");
			}
		}
	}

	@Inject
	public void setLogger(@Named("applicationLog") Log logger)
	{
		this.applicationLog = logger;
	}

	public action_code process(ETLTestOperation etlTestOperation, ETLTestValueObject etlTestValueObject, VariableContext context, ExecutionContext econtext) throws TestAssertionFailure, TestExecutionError, TestWarning {
		InformaticaRepository informaticaRepository = informaticaFeatureModule.repositoryForRequest(etlTestValueObject);

		if (etlTestOperation.getOperationName().equals("createInformaticaFolder"))
		{
			InformaticaRepositoryClient ipc = informaticaRepository.getInformaticaRepositoryClient();

			try {
				ipc.createFolder(etlTestValueObject.queryRequired("folder").getValueAsString());
			} catch (Exception e) {
				applicationLog.severe("Failed to execute Informatica Command", e);
				throw new TestExecutionError("Failed to execute Informatica Command", InformaticaConstants.ERR_CREATE_FOLDER, e);
			}

			return action_code.handled;
		}

		if (etlTestOperation.getOperationName().equals("deleteInformaticaFolder"))
		{
			InformaticaRepositoryClient ipc = informaticaRepository.getInformaticaRepositoryClient();

			try {
				ipc.deleteFolder(etlTestValueObject.queryRequired("folder").getValueAsString());
			} catch (Exception e) {
				throw new TestExecutionError("Failed to execute Informatica Command", InformaticaConstants.ERR_DELETE_FOLDER, e);
			}

			return action_code.handled;
		}

		if (etlTestOperation.getOperationName().equals("createInformaticaConnection"))
		{
			InformaticaRepositoryClient ipc = informaticaRepository.getInformaticaRepositoryClient();

			try {
				ipc.createConnection(
					etlTestValueObject.queryRequired("serverName").getValueAsString(),
					etlTestValueObject.queryRequired("databaseName").getValueAsString(),
					etlTestValueObject.queryRequired("databaseUserName").getValueAsString(),
					etlTestValueObject.queryRequired("databasePassword").getValueAsString(),
					etlTestValueObject.queryRequired("relationalConnectionName").getValueAsString()
				);
			} catch (Exception e) {
				throw new TestExecutionError("Failed to execute Informatica Command", InformaticaConstants.ERR_CREATE_CONNECTION, e);
			}

			return action_code.handled;
		}

		if (etlTestOperation.getOperationName().equals("deleteInformaticaConnection"))
		{
			InformaticaRepositoryClient ipc = informaticaRepository.getInformaticaRepositoryClient();

			try {
				ipc.deleteConnection(
					etlTestValueObject.queryRequired("relationalConnectionName").getValueAsString()
				);
			} catch (Exception e) {
				throw new TestExecutionError("Failed to execute Informatica Command", InformaticaConstants.ERR_DELETE_CONNECTION, e);
			}

			return action_code.handled;
		}

		return action_code.defer;
	}

	@Override
	public action_code execute(ExecuteRequest request, ETLTestOperation testOperation, ETLTestValueObject valueObject, VariableContext variableContext, ExecutionContext executionContext)
			throws TestAssertionFailure, TestExecutionError, TestWarning
	{
		return informaticaExecutor.execute(request, testOperation, valueObject, variableContext, executionContext);
	}
}